@@ -18,20 +18,21 @@ So, in summary: Col objects are what you define, but SOCol objects
 are what gets used.
 """
 
-import re, time
 from array import array
+from itertools import count
+import re, time
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-import sqlbuilder
+import weakref
+from formencode import compound, validators
+from classregistry import findClass
 # Sadly the name "constraints" conflicts with many of the function
 # arguments in this module, so we rename it:
-import constraints as consts
-from formencode import compound
-from formencode import validators
-from classregistry import findClass
-from itertools import count
+import constraints as constrs
+import sqlbuilder
+from styles import capword
 
 NoDefault = sqlbuilder.NoDefault
 
@@ -72,20 +73,6 @@ if mxdatetime_available:
 
 creationOrder = count()
 
-class SQLValidator(compound.All):
-    def attemptConvert(self, value, state, validate):
-        if validate is validators.to_python:
-            vlist = list(self.validators[:])
-            vlist.reverse()
-        elif validate is validators.from_python:
-            vlist = self.validators
-        else:
-            raise RuntimeError
-        for validator in vlist:
-            value = validate(validator, value, state)
-        return value
-
-
 ########################################
 ## Columns
 ########################################
@@ -111,6 +98,7 @@ class SOCol(object):
                  sqlType=None,
                  columnDef=None,
                  validator=None,
+                 validator2=None,
                  immutable=False,
                  cascade=None,
                  lazy=False,
@@ -119,6 +107,7 @@ class SOCol(object):
                  title=None,
                  tags=[],
                  origName=None,
+                 dbEncoding=None,
                  extra_vars=None):
 
         super(SOCol, self).__init__()
@@ -162,7 +151,7 @@ class SOCol(object):
         elif notNone is not NoDefault:
             self.notNone = notNone
         if self.notNone:
-            self.constraints = [consts.notNull] + self.constraints
+            self.constraints = [constrs.notNull] + self.constraints
 
         self.name = name
         self.soClass = soClass
@@ -196,22 +185,29 @@ class SOCol(object):
         # alternateID means that this is a unique column that
         # can be used to identify rows
         self.alternateID = alternateID
-        if self.alternateID and alternateMethodName is None:
-            self.alternateMethodName = 'by' + self.name[0].capitalize() + self.name[1:]
-        else:
-            self.alternateMethodName = alternateMethodName
 
         if unique is NoDefault:
             self.unique = alternateID
         else:
             self.unique = unique
+        if self.unique and alternateMethodName is None:
+            self.alternateMethodName = 'by' + capword(self.name)
+        else:
+            self.alternateMethodName = alternateMethodName
 
         _validators = self.createValidators()
-        if _validators:
-            if validator: _validators.append(validator)
-            self.validator = SQLValidator.join(_validators[0], *_validators[1:])
-        else:
-            self.validator = validator
+        if validator: _validators.append(validator)
+        if validator2: _validators.insert(0, validator2)
+        _vlen = len(_validators)
+        if _vlen:
+            for _validator in _validators:
+                _validator.soCol=weakref.proxy(self)
+        if _vlen == 0:
+            self.validator = None # Set sef.{from,to}_python
+        elif _vlen == 1:
+            self.validator = _validators[0]
+        elif _vlen > 1:
+            self.validator = compound.All.join(_validators[0], *_validators[1:])
         self.noCache = noCache
         self.lazy = lazy
         # this is in case of ForeignKey, where we rename the column
@@ -219,6 +215,7 @@ class SOCol(object):
         self.origName = origName or name
         self.title = title
         self.tags = tags
+        self.dbEncoding = dbEncoding
 
         if extra_vars:
             for name, value in extra_vars.items():
@@ -353,8 +350,9 @@ class SOCol(object):
         if obj is None:
             # class attribute, return the descriptor itself
             return self
-        if obj.sqlmeta.obsolete:
-            raise '@@: figure out the exception for a delete'
+        if obj.sqlmeta._obsolete:
+            raise RuntimeError('The object <%s %s> is obsolete' % (
+                obj.__class__.__name__, obj.id))
         if obj.sqlmeta.cacheColumns:
             columns = obj.sqlmeta._columnCache
             if columns is None:
@@ -376,6 +374,22 @@ class SOCol(object):
     def __delete__(self, obj):
         raise AttributeError("I can't be deleted from %r" % obj)
 
+    def getDbEncoding(self, state, default='utf-8'):
+        if self.dbEncoding:
+            return self.dbEncoding
+        dbEncoding = state.soObject.sqlmeta.dbEncoding
+        if dbEncoding:
+            return dbEncoding
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbEncoding = None
+        else:
+            dbEncoding = getattr(connection, "dbEncoding", None)
+        if not dbEncoding:
+            dbEncoding = default
+        return dbEncoding
+
 
 class Col(object):
 
@@ -418,6 +432,13 @@ class Col(object):
             self._name or '(unnamed)')
 
 
+class SOValidator(validators.Validator):
+    def getDbEncoding(self, state, default='utf-8'):
+        try:
+            return self.dbEncoding
+        except AttributeError:
+            return self.soCol.getDbEncoding(state, default=default)
+
 
 class SOStringLikeCol(SOCol):
     """A common ancestor for SOStringCol and SOUnicodeCol"""
@@ -435,9 +456,9 @@ class SOStringLikeCol(SOCol):
         super(SOStringLikeCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        constraints = [consts.isString]
+        constraints = [constrs.isString]
         if self.length is not None:
-            constraints += [consts.MaxLength(self.length)]
+            constraints += [constrs.MaxLength(self.length)]
         return constraints
 
     def _sqlType(self):
@@ -466,7 +487,7 @@ class SOStringLikeCol(SOCol):
 
     def _sqliteType(self):
         self._check_case_sensitive("SQLite")
-        return super(SOStringLikeCol, self)._sqliteType().replace("CHAR(", "CHAR (")
+        return super(SOStringLikeCol, self)._sqliteType()
 
     def _sybaseType(self):
         self._check_case_sensitive("SYBASE")
@@ -482,7 +503,7 @@ class SOStringLikeCol(SOCol):
             if self.connection and self.connection.can_use_max_types():
                 type = 'VARCHAR(MAX)'
             else:
-                type = 'varchar(4000)'
+                type = 'VARCHAR(4000)'
         elif self.varchar:
             type = 'VARCHAR(%i)' % self.length
         else:
@@ -506,88 +527,116 @@ class SOStringLikeCol(SOCol):
             return self._sqlType()
 
 
-class StringValidator(validators.Validator):
+class StringValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
             return None
+        try:
+            connection = state.connection or state.soObject._connection
+            binaryType = connection._binaryType
+        except AttributeError:
+            binaryType = type(None) # Just a simple workaround
+        dbEncoding = self.getDbEncoding(state, default='ascii')
         if isinstance(value, unicode):
-            connection = state.soObject._connection
-            dbEncoding = getattr(connection, "dbEncoding", "ascii")
             return value.encode(dbEncoding)
-        return value
-
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if isinstance(value, str):
+        if self.dataType and isinstance(value, self.dataType):
             return value
-        if isinstance(value, unicode):
-            connection = state.soObject._connection
-            dbEncoding = getattr(connection, "dbEncoding", "ascii")
-            return value.encode(dbEncoding)
-        return value
+        if isinstance(value, (str, buffer, binaryType, sqlbuilder.SQLExpression)):
+            return value
+        if hasattr(value, '__unicode__'):
+            return unicode(value).encode(dbEncoding)
+        raise validators.Invalid("expected a str in the StringCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
+
+    from_python = to_python
 
 class SOStringCol(SOStringLikeCol):
 
-    def createValidators(self):
-        return [StringValidator()] + \
+    def createValidators(self, dataType=None):
+        return [StringValidator(name=self.name, dataType=dataType)] + \
             super(SOStringCol, self).createValidators()
 
 class StringCol(Col):
     baseClass = SOStringCol
 
-class UnicodeStringValidator(validators.Validator):
+
+class NQuoted(sqlbuilder.SQLExpression):
+    def __init__(self, value):
+        assert isinstance(value, unicode)
+        self.value = value
+    def __hash__(self):
+        return hash(self.value)
+    def __sqlrepr__(self, db):
+        assert db == 'mssql'
+        return "N" + sqlbuilder.sqlrepr(self.value, db)
+
+class UnicodeStringValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, unicode):
+        if isinstance(value, (unicode, sqlbuilder.SQLExpression)):
             return value
+        if isinstance(value, str):
+            return unicode(value, self.getDbEncoding(state))
         if isinstance(value, array): # MySQL
-            return unicode(value.tostring(), self.db_encoding)
-        return unicode(value, self.db_encoding)
+            return unicode(value.tostring(), self.getDbEncoding(state))
+        if hasattr(value, '__unicode__'):
+            return unicode(value)
+        raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
     def from_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, str):
+        if isinstance(value, (str, sqlbuilder.SQLExpression)):
             return value
-        return value.encode(self.db_encoding)
+        if isinstance(value, unicode):
+            try:
+                connection = state.connection or state.soObject._connection
+            except AttributeError:
+                pass
+            else:
+                if connection.dbName == 'mssql':
+                    return NQuoted(value)
+            return value.encode(self.getDbEncoding(state))
+        if hasattr(value, '__unicode__'):
+            return unicode(value).encode(self.getDbEncoding(state))
+        raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
 class SOUnicodeCol(SOStringLikeCol):
-    def __init__(self, **kw):
-        self.dbEncoding = kw.pop('dbEncoding', 'UTF-8')
-        super(SOUnicodeCol, self).__init__(**kw)
+    def _mssqlType(self):
+        if self.customSQLType is not None:
+            return self.customSQLType
+        return 'N' + super(SOUnicodeCol, self)._mssqlType()
 
     def createValidators(self):
-        return [UnicodeStringValidator(db_encoding=self.dbEncoding)] + \
+        return [UnicodeStringValidator(name=self.name)] + \
             super(SOUnicodeCol, self).createValidators()
 
 class UnicodeCol(Col):
     baseClass = SOUnicodeCol
 
 
-class IntValidator(validators.Validator):
+class IntValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
             return None
         if isinstance(value, (int, long, sqlbuilder.SQLExpression)):
             return value
-        try:
-            return int(value)
-        except:
-            raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
+        for converter, attr_name in (int, '__int__'), (long, '__long__'):
+            if hasattr(value, attr_name):
+                try:
+                    return converter(value)
+                except:
+                    break
+        raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
                 (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if not isinstance(value, (int, long, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+    from_python = to_python
 
 class SOIntCol(SOCol):
     # 3-03 @@: support precision, maybe max and min directly
@@ -598,7 +647,7 @@ class SOIntCol(SOCol):
         SOCol.__init__(self, **kw)
 
     def autoConstraints(self):
-        return [consts.isInt]
+        return [constrs.isInt]
 
     def createValidators(self):
         return [IntValidator(name=self.name)] + \
@@ -652,30 +701,26 @@ class BigIntCol(Col):
     baseClass = SOBigIntCol
 
 
-class BoolValidator(validators.Validator):
+class BoolValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
             return None
-        elif not value:
-            return False
-        else:
-            return True
+        if isinstance(value, (bool, sqlbuilder.SQLExpression)):
+            return value
+        if isinstance(value, (int, long)) or hasattr(value, '__nonzero__'):
+            return bool(value)
+        raise validators.Invalid("expected a bool or an int in the BoolCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        elif value:
-            return True
-        else:
-            return False
+    from_python = to_python
 
 class SOBoolCol(SOCol):
     def autoConstraints(self):
-        return [consts.isBool]
+        return [constrs.isBool]
 
     def createValidators(self):
-        return [BoolValidator()] + \
+        return [BoolValidator(name=self.name)] + \
             super(SOBoolCol, self).createValidators()
 
     def _postgresType(self):
@@ -703,32 +748,29 @@ class BoolCol(Col):
     baseClass = SOBoolCol
 
 
-class FloatValidator(validators.Validator):
+class FloatValidator(SOValidator):
 
     def to_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, (int, long, float, sqlbuilder.SQLExpression)):
+        if isinstance(value, (float, int, long, sqlbuilder.SQLExpression)):
             return value
-        try:
-            return float(value)
-        except:
-            raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
+        for converter, attr_name in  (float, '__float__'), (int, '__int__'), (long, '__long__'):
+            if hasattr(value, attr_name):
+                try:
+                    return converter(value)
+                except:
+                    break
+        raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if not isinstance(value, (int, long, float, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+    from_python = to_python
 
 class SOFloatCol(SOCol):
     # 3-03 @@: support precision (e.g., DECIMAL)
 
     def autoConstraints(self):
-        return [consts.isFloat]
+        return [constrs.isFloat]
 
     def createValidators(self):
         return [FloatValidator(name=self.name)] + \
@@ -750,6 +792,10 @@ class SOKeyCol(SOCol):
     # 3-03 @@: this should have a simplified constructor
     # Should provide foreign key information for other DBs.
 
+    def __init__(self, **kw):
+        self.refColumn = kw.pop('refColumn', None)
+        super(SOKeyCol, self).__init__(**kw)
+
     def _sqlType(self):
         return self.key_type[self.soClass.sqlmeta.idType]
 
@@ -770,18 +816,18 @@ class SOForeignKey(SOKeyCol):
     def __init__(self, **kw):
         foreignKey = kw['foreignKey']
         style = kw['soClass'].sqlmeta.style
-        if not kw.get('name'):
-            kw['name'] = style.instanceAttrToIDAttr(style.pythonClassToAttr(foreignKey))
-        else:
+        if kw.get('name'):
             kw['origName'] = kw['name']
             kw['name'] = style.instanceAttrToIDAttr(kw['name'])
+        else:
+            kw['name'] = style.instanceAttrToIDAttr(style.pythonClassToAttr(foreignKey))
         super(SOForeignKey, self).__init__(**kw)
 
     def sqliteCreateSQL(self):
         sql = SOKeyCol.sqliteCreateSQL(self)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -810,7 +856,7 @@ class SOForeignKey(SOKeyCol):
         sTName = self.soClass.sqlmeta.table
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -836,7 +882,7 @@ class SOForeignKey(SOKeyCol):
         sTLocalName = sTName.split('.')[-1]
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -865,7 +911,7 @@ class SOForeignKey(SOKeyCol):
         sql = SOKeyCol.sybaseCreateSQL(self)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
                      {'tName':tName,
                       'idName':idName})
@@ -880,7 +926,7 @@ class SOForeignKey(SOKeyCol):
         sql = SOKeyCol.mssqlCreateSQL(self, connection)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
                      {'tName':tName,
                       'idName':idName})
@@ -897,7 +943,7 @@ class SOForeignKey(SOKeyCol):
         #I assume that foreign key name is identical to the id of the reference table
         sql = ' '.join([fidName, self._maxdbType()])
         tName = other.sqlmeta.table
-        idName  = other.sqlmeta.idName
+        idName  = self.refColumn or other.sqlmeta.idName
         sql=sql + ',' + '\n'
         sql=sql + 'FOREIGN KEY (%s) REFERENCES %s(%s)'%(fidName,tName,idName)
         return sql
@@ -914,19 +960,20 @@ class ForeignKey(KeyCol):
         super(ForeignKey, self).__init__(foreignKey=foreignKey, **kw)
 
 
-class EnumValidator(validators.Validator):
+class EnumValidator(SOValidator):
 
     def to_python(self, value, state):
         if value in self.enumValues:
+            if isinstance(value, unicode):
+                dbEncoding = self.getDbEncoding(state)
+                value = value.encode(dbEncoding)
             return value
         elif not self.notNone and value is None:
             return None
-        else:
-            raise validators.Invalid("expected a member of %r in the EnumCol '%s', got %r instead" % \
-                                     (self.enumValues, self.name, value), value, state)
+        raise validators.Invalid("expected a member of %r in the EnumCol '%s', got %r instead" % \
+            (self.enumValues, self.name, value), value, state)
 
-    def from_python(self, value, state):
-        return self.to_python(value, state)
+    from_python = to_python
 
 class SOEnumCol(SOCol):
 
@@ -937,7 +984,7 @@ class SOEnumCol(SOCol):
         super(SOEnumCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        return [consts.isString, consts.InList(self.enumValues)]
+        return [constrs.isString, constrs.InList(self.enumValues)]
 
     def createValidators(self):
         return [EnumValidator(name=self.name, enumValues=self.enumValues,
@@ -958,8 +1005,7 @@ class SOEnumCol(SOCol):
         checkConstraint = "CHECK (%s in (%s))" % (self.dbName, enumValues)
         return "VARCHAR(%i) %s" % (length, checkConstraint)
 
-    def _sqliteType(self):
-        return self._postgresType().replace("CHAR(", "CHAR (")
+    _sqliteType = _postgresType
 
     def _sybaseType(self):
         return self._postgresType()
@@ -975,7 +1021,7 @@ class SOEnumCol(SOCol):
         return "VARCHAR(%i)" % (length), checkConstraint
 
     def _maxdbType(self):
-        raise "Enum type is not supported"
+        raise TypeError("Enum type is not supported on MAX DB")
 
     def _getlength(self, obj):
         """
@@ -990,21 +1036,25 @@ class EnumCol(Col):
     baseClass = SOEnumCol
 
 
-class SetValidator(validators.Validator):
+class SetValidator(SOValidator):
     """
     Translates Python tuples into SQL comma-delimited SET strings.
     """
 
     def to_python(self, value, state):
-        if not isinstance(value, str):
-            raise validators.Invalid("expected a value string, got % instead" % \
-                                      (value), value, state)
-        return tuple(value.split(","))
+        if isinstance(value, str):
+            return tuple(value.split(","))
+        raise validators.Invalid("expected a string in the SetCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
     def from_python(self, value, state):
-        if not isinstance(value, tuple):
+        if isinstance(value, basestring):
             value = (value,)
-        return ",".join([v for v in value])
+        try:
+            return ",".join(value)
+        except:
+            raise validators.Invalid("expected a string or a sequence of stringsin the SetCol '%s', got %s %r instead" % \
+                (self.name, type(value), value), value, state)
 
 class SOSetCol(SOCol):
     def __init__(self, **kw):
@@ -1014,7 +1064,7 @@ class SOSetCol(SOCol):
         super(SOSetCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        return [consts.isString, consts.InList(self.setValues)]
+        return [constrs.isString, constrs.InList(self.setValues)]
 
     def createValidators(self):
         return [SetValidator(name=self.name, setValues=self.setValues)] + \
@@ -1137,6 +1187,7 @@ class SODateTimeCol(SOCol):
 
 class DateTimeCol(Col):
     baseClass = SODateTimeCol
+    @staticmethod
     def now():
         if default_datetime_implementation == DATETIME_IMPLEMENTATION:
             return datetime.datetime.now()
@@ -1146,14 +1197,13 @@ class DateTimeCol(Col):
             assert 0, ("No datetime implementation available "
                        "(DATETIME_IMPLEMENTATION=%r)"
                        % DATETIME_IMPLEMENTATION)
-    now = staticmethod(now)
 
 
 class DateValidator(DateTimeValidator):
     def to_python(self, value, state):
         if isinstance(value, datetime.datetime):
             value = value.date()
-        if isinstance(value, datetime.date):
+        if isinstance(value, (datetime.date, sqlbuilder.SQLExpression)):
             return value
         value = super(DateValidator, self).to_python(value, state)
         if isinstance(value, datetime.datetime):
@@ -1211,7 +1261,7 @@ class DateCol(Col):
 
 class TimeValidator(DateTimeValidator):
     def to_python(self, value, state):
-        if isinstance(value, datetime.time):
+        if isinstance(value, (datetime.time, sqlbuilder.SQLExpression)):
             return value
         if isinstance(value, datetime.timedelta):
             if value.days:
@@ -1224,6 +1274,8 @@ class TimeValidator(DateTimeValidator):
             value = value.time()
         return value
 
+    from_python = to_python
+
 class SOTimeCol(SOCol):
     timeFormat = '%H:%M:%S'
 
@@ -1271,6 +1323,8 @@ class SOTimestampCol(SODateTimeCol):
     """
 
     def __init__(self, **kw):
+        if 'default' not in kw:
+            kw['default'] = None
         SOCol.__init__(self, **kw)
 
     def _mysqlType(self):
@@ -1280,9 +1334,27 @@ class TimestampCol(Col):
     baseClass = SOTimestampCol
 
 
+class TimedeltaValidator(SOValidator):
+    def to_python(self, value, state):
+        return value
+
+    from_python = to_python
+
+class SOTimedeltaCol(SOCol):
+    def _postgresType(self):
+        return 'INTERVAL'
+
+    def createValidators(self):
+        return [TimedeltaValidator(name=self.name)] + \
+            super(SOTimedeltaCol, self).createValidators()
+
+class TimedeltaCol(Col):
+    baseClass = SOTimedeltaCol
+
+
 from decimal import Decimal
 
-class DecimalValidator(validators.Validator):
+class DecimalValidator(SOValidator):
     def to_python(self, value, state):
         if value is None:
             return None
@@ -1290,9 +1362,13 @@ class DecimalValidator(validators.Validator):
             return value
         if isinstance(value, float):
             value = str(value)
-        connection = state.soObject._connection
-        if hasattr(connection, "decimalSeparator"):
-            value = value.replace(connection.decimalSeparator, ".")
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            pass
+        else:
+            if hasattr(connection, "decimalSeparator"):
+                value = value.replace(connection.decimalSeparator, ".")
         try:
             return Decimal(value)
         except:
@@ -1305,18 +1381,22 @@ class DecimalValidator(validators.Validator):
         if isinstance(value, float):
             value = str(value)
         if isinstance(value, basestring):
-            connection = state.soObject._connection
-            if hasattr(connection, "decimalSeparator"):
-                value = value.replace(connection.decimalSeparator, ".")
+            try:
+                connection = state.connection or state.soObject._connection
+            except AttributeError:
+                pass
+            else:
+                if hasattr(connection, "decimalSeparator"):
+                    value = value.replace(connection.decimalSeparator, ".")
             try:
                 return Decimal(value)
             except:
                 raise validators.Invalid("can not parse Decimal value '%s' in the DecimalCol from '%s'" %
                     (value, getattr(state, 'soObject', '(unknown)')), value, state)
-        if not isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected a decimal in the DecimalCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+        if isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
+            return value
+        raise validators.Invalid("expected a Decimal in the DecimalCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
 class SODecimalCol(SOCol):
 
@@ -1387,18 +1467,19 @@ class SODecimalStringCol(SOStringCol):
 
     def createValidators(self):
         if self.quantize:
-            v = DecimalStringValidator(
+            v = DecimalStringValidator(name=self.name,
                 precision=Decimal(10) ** (-1 * int(self.precision)),
                 max=Decimal(10) ** (int(self.size) - int(self.precision)))
         else:
-            v = DecimalStringValidator(precision=0)
-        return [v] + super(SODecimalStringCol, self).createValidators()
+            v = DecimalStringValidator(name=self.name, precision=0)
+        return [v] + \
+            super(SODecimalStringCol, self).createValidators(dataType=Decimal)
 
 class DecimalStringCol(StringCol):
     baseClass = SODecimalStringCol
 
 
-class BinaryValidator(validators.Validator):
+class BinaryValidator(SOValidator):
     """
     Validator for binary types.
 
@@ -1411,12 +1492,19 @@ class BinaryValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbName = None
+            binaryType = type(None) # Just a simple workaround
+        else:
+            dbName = connection.dbName
+            binaryType = connection._binaryType
         if isinstance(value, str):
-            connection = state.soObject._connection
-            if connection.dbName == "sqlite":
+            if dbName == "sqlite":
                 value = connection.module.decode(value)
             return value
-        if isinstance(value, (buffer, state.soObject._connection._binaryType)):
+        if isinstance(value, (buffer, binaryType)):
             cachedValue = self._cachedValue
             if cachedValue and cachedValue[1] == value:
                 return cachedValue[0]
@@ -1429,7 +1517,8 @@ class BinaryValidator(validators.Validator):
     def from_python(self, value, state):
         if value is None:
             return None
-        binary = state.soObject._connection.createBinary(value)
+        connection = state.connection or state.soObject._connection
+        binary = connection.createBinary(value)
         self._cachedValue = (value, binary)
         return binary
 
@@ -1481,8 +1570,7 @@ class PickleValidator(BinaryValidator):
         if value is None:
             return None
         if isinstance(value, unicode):
-            connection = state.soObject._connection
-            dbEncoding = getattr(connection, "dbEncoding", "ascii")
+            dbEncoding = self.getDbEncoding(state, default='ascii')
             value = value.encode(dbEncoding)
         if isinstance(value, str):
             return pickle.loads(value)
@@ -1501,8 +1589,8 @@ class SOPickleCol(SOBLOBCol):
         super(SOPickleCol, self).__init__(**kw)
 
     def createValidators(self):
-        return [PickleValidator(
-            name=self.name, pickleProtocol=self.pickleProtocol)] + \
+        return [PickleValidator(name=self.name,
+                pickleProtocol=self.pickleProtocol)] + \
             super(SOPickleCol, self).createValidators()
 
     def _mysqlType(self):
@@ -1518,7 +1606,7 @@ class PickleCol(BLOBCol):
 
 
 def pushKey(kw, name, value):
-    if not kw.has_key(name):
+    if not name in kw:
         kw[name] = value
 
 all = []
