@@ -8,26 +8,28 @@ class MSSQLConnection(DBAPI):
     dbName = 'mssql'
     schemes = [dbName]
 
-    def __init__(self, db, user, password='', host='localhost',
+    limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
+
+    def __init__(self, db, user, password='', host='localhost', port=None,
                  autoCommit=0, **kw):
-        backends = kw.pop('backend', None) or 'adodb,pymssql'
-        for backend in backends.split(','):
-            backend = backend.strip()
-            if not backend:
+        drivers = kw.pop('driver', None) or 'adodb,pymssql'
+        for driver in drivers.split(','):
+            driver = driver.strip()
+            if not driver:
                 continue
             try:
-                if backend in ('adodb', 'adodbapi'):
+                if driver in ('adodb', 'adodbapi'):
                     import adodbapi as sqlmodule
-                elif backend == 'pymssql':
+                elif driver == 'pymssql':
                     import pymssql as sqlmodule
                 else:
-                    raise ValueError('Unknown MSSQL backend "%s", expected adodb or pymssql' % backend)
+                    raise ValueError('Unknown MSSQL driver "%s", expected adodb or pymssql' % driver)
             except ImportError:
                 pass
             else:
                 break
         else:
-            raise ImportError('Cannot find an MSSQL backend, tried %s' % backends)
+            raise ImportError('Cannot find an MSSQL driver, tried %s' % drivers)
         self.module = sqlmodule
 
         if sqlmodule.__name__ == 'adodbapi':
@@ -46,10 +48,10 @@ class MSSQLConnection(DBAPI):
             # MSDE does not allow SQL server login 
             if kw.get("sspi"):
                 conn_str += "Integrated Security=SSPI;Persist Security Info=False"
-                self.make_conn_str = lambda keys: [conn_str % (keys.host, keys.db)]
+                self.make_conn_str = lambda keys: conn_str % (keys.host, keys.db)
             else:
                 conn_str += "User Id=%s;Password=%s"
-                self.make_conn_str = lambda keys: [conn_str % (keys.host, keys.db, keys.user, keys.password)]
+                self.make_conn_str = lambda keys: conn_str % (keys.host, keys.db, keys.user, keys.password)
 
             kw.pop("sspi", None)
             kw.pop("ncli", None)
@@ -59,25 +61,33 @@ class MSSQLConnection(DBAPI):
             sqlmodule.Binary = lambda st: str(st)
             # don't know whether pymssql uses unicode
             self.usingUnicodeStrings = False
-            self.make_conn_str = lambda keys:  \
-                   ["", keys.user, keys.password, keys.host, keys.db]
+            def _make_conn_str(keys):
+                keys_dict = {}
+                for attr, value in (
+                    ('user', keys.user),
+                    ('password', keys.password),
+                    ('host', keys.host),
+                    ('port', keys.port),
+                    ('database', keys.db),
+                ):
+                    if value: keys_dict[attr] = value
+                return keys_dict
+            self.make_conn_str = _make_conn_str
 
         self.autoCommit=int(autoCommit)
-        self.host = host
-        self.db = db
         self.user = user
         self.password = password
-        self.limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
-        self.password = password
+        self.host = host
+        self.port = port
+        self.db = db
         self._can_use_max_types = None
         DBAPI.__init__(self, **kw)
 
-    def connectionFromURI(cls, uri):
-        user, password, host, port, path, args = cls._parseURI(uri)
+    @classmethod
+    def _connectionFromParams(cls, user, password, host, port, path, args):
         path = path.strip('/')
-        return cls(user=user, password=password, host=host or 'localhost',
-                   db=path, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+        return cls(user=user, password=password,
+                   host=host or 'localhost', port=port, db=path, **args)
 
     def insert_id(self, conn):
         """
@@ -90,7 +100,11 @@ class MSSQLConnection(DBAPI):
         return c.fetchone()[0]
 
     def makeConnection(self):
-        con = self.dbconnection( *self.make_conn_str(self) )
+        conn_descr = self.make_conn_str(self)
+        if isinstance(conn_descr, dict):
+            con = self.dbconnection(**conn_descr)
+        else:
+            con = self.dbconnection(conn_descr)
         cur = con.cursor()
         cur.execute('SET ANSI_NULLS ON')
         cur.execute("SELECT CAST('12345.21' AS DECIMAL(10, 2))")
@@ -99,13 +113,10 @@ class MSSQLConnection(DBAPI):
         return con
 
     HAS_IDENTITY = """
-       SELECT col.name, col.status, obj.name
-       FROM syscolumns col
-       JOIN sysobjects obj
-       ON obj.id = col.id
-       WHERE obj.name = '%s'
-       and col.autoval is not null
-
+       select 1
+       from INFORMATION_SCHEMA.COLUMNS
+       where TABLE_NAME = '%s'
+       and COLUMNPROPERTY(object_id(TABLE_NAME), COLUMN_NAME, 'IsIdentity') = 1
     """
     def _hasIdentity(self, conn, table):
         query = self.HAS_IDENTITY % table
@@ -142,7 +153,7 @@ class MSSQLConnection(DBAPI):
 
         q = self._insertSQL(table, names, values)
         if self.debug:
-            print 'QueryIns: %s' % q
+            self.printDebug(conn, q, 'QueryIns')
         c.execute(q)
         if has_identity:
             c.execute('SET IDENTITY_INSERT %s OFF' % table)
@@ -153,6 +164,7 @@ class MSSQLConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
+    @classmethod
     def _queryAddLimitOffset(cls, query, start, end):
         if end and not start:
             limit_str = "SELECT TOP %i" % end
@@ -162,7 +174,6 @@ class MSSQLConnection(DBAPI):
                 return ' '.join([limit_str, match.group(2)])
         else:
             return query
-    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createReferenceConstraint(self, soClass, col):
         return col.mssqlCreateReferenceConstraint()
