@@ -21,7 +21,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 Instructions
@@ -49,9 +49,7 @@ objects, as well as the proper SQL representation of basic Python
 types (None==NULL).
 
 There are a number of DB-specific SQL features that this does not
-implement.  There are a bunch of normal ANSI features also not present
--- particularly left joins, among others.  You may wish to only use
-this to generate ``WHERE`` clauses.
+implement.  There are a bunch of normal ANSI features also not present.
 
 See the bottom of this module for some examples, and run it (i.e.
 ``python sql.py``) to see the results of those examples.
@@ -70,7 +68,7 @@ import types
 import weakref
 
 import classregistry
-from converters import sqlrepr, registerConverter
+from converters import registerConverter, sqlrepr, quote_str, unquote_str
 
 
 class VersionError(Exception):
@@ -80,8 +78,9 @@ class NoDefault:
 
 
 class SQLObjectState(object):
-    def __init__(self, soObject):
+    def __init__(self, soObject, connection=None):
         self.soObject = weakref.proxy(soObject)
+        self.connection = connection
 
 
 safeSQLRE = re.compile(r'^[a-zA-Z_][a-zA-Z0-9_\.]*$')
@@ -96,6 +95,12 @@ def execute(expr, executor):
     else:
         return expr
 
+
+def _str_or_sqlrepr(expr, db):
+    if isinstance(expr, basestring):
+        return expr
+    return sqlrepr(expr, db)
+
 ########################################
 ## Expression generation
 ########################################
@@ -128,7 +133,7 @@ class SQLExpression:
     def __abs__(self):
         return SQLConstant("ABS")(self)
     def __mod__(self, other):
-        return SQLConstant("MOD")(self, other)
+        return SQLModulo(self, other)
     def __rmod__(self, other):
         return SQLConstant("MOD")(other, self)
 
@@ -260,7 +265,18 @@ class SQLOp(SQLExpression):
             return operatorMap[self.op.upper()](execute(self.expr1, executor),
                                                 execute(self.expr2, executor))
 
+class SQLModulo(SQLOp):
+    def __init__(self, expr1, expr2):
+        SQLOp.__init__(self, '%', expr1, expr2)
+    def __sqlrepr__(self, db):
+        if db == 'sqlite':
+            return SQLOp.__sqlrepr__(self, db)
+        s1 = sqlrepr(self.expr1, db)
+        s2 = sqlrepr(self.expr2, db)
+        return "MOD(%s, %s)" % (s1, s2)
+
 registerConverter(SQLOp, SQLExprConverter)
+registerConverter(SQLModulo, SQLExprConverter)
 
 class SQLCall(SQLExpression):
     def __init__(self, expr, args):
@@ -373,9 +389,7 @@ class Table(SQLExpression):
             raise AttributeError
         return self.FieldClass(self.tableName, attr)
     def __sqlrepr__(self, db):
-        if isinstance(self.tableName, str):
-            return self.tableName
-        return sqlrepr(self.tableName, db)
+        return _str_or_sqlrepr(self.tableName, db)
     def execute(self, executor):
         raise ValueError, "Tables don't have values"
 
@@ -413,7 +427,7 @@ class SQLObjectTable(Table):
 class SQLObjectTableWithJoins(SQLObjectTable):
 
     def __getattr__(self, attr):
-        if attr+'ID' in [k for (k,v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
+        if attr+'ID' in [k for (k, v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
             column = self.soClass.sqlmeta.columns[attr+'ID']
             return self._getattrFromForeignKey(column, attr)
         elif attr in [x.joinMethodName for x in self.soClass.sqlmeta.joins]:
@@ -478,7 +492,7 @@ class AliasTable(Table):
     def __init__(self, table, alias=None):
         if hasattr(table, "sqlmeta"):
             tableName = SQLConstant(table.sqlmeta.table)
-        elif isinstance(table, (Select,Union)):
+        elif isinstance(table, (Select, Union)):
             assert alias is not None, "Alias name cannot be constructed from Select instances, please provide 'alias' kw."
             tableName = Subquery('', table)
             table = None
@@ -528,7 +542,7 @@ class Union(SQLExpression):
                 if isinstance(t, Table):
                     t = t.tableName
                 if not isinstance(t, SQLExpression):
-                    t = SQLConstant(t.sqlmeta.table)
+                    t = SQLConstant(t)
             tabs.append(t)
         self.tables = tabs
 
@@ -604,12 +618,7 @@ class Select(SQLExpression):
         clause = self.ops['clause']
         if isinstance(clause, basestring):
             clause = SQLConstant('(%s)' % clause)
-
-        if clause == SQLTrueClause:
-            newClause = filter_clause
-        else:
-            newClause = AND(clause, filter_clause)
-        return self.newClause(newClause)
+        return self.newClause(AND(clause, filter_clause))
 
     def __sqlrepr__(self, db):
 
@@ -617,11 +626,11 @@ class Select(SQLExpression):
         if self.ops['distinct']:
             select += " DISTINCT"
             if self.ops['distinctOn'] is not NoDefault:
-                select += " ON(%s)" % sqlrepr(self.ops['distinctOn'], db)
+                select += " ON(%s)" % _str_or_sqlrepr(self.ops['distinctOn'], db)
         if not self.ops['lazyColumns']:
-            select += " %s" % ", ".join([str(sqlrepr(v, db)) for v in self.ops['items']])
+            select += " %s" % ", ".join([str(_str_or_sqlrepr(v, db)) for v in self.ops['items']])
         else:
-            select += " %s" % sqlrepr(self.ops['items'][0], db)
+            select += " %s" % _str_or_sqlrepr(self.ops['items'][0], db)
 
         join = []
         join_str = ''
@@ -645,8 +654,9 @@ class Select(SQLExpression):
             if isinstance(thing, SQLExpression):
                 tables.update(tablesUsedSet(thing, db))
         for j in join:
-            t1, t2 = sqlrepr(j.table1, db), sqlrepr(j.table2, db)
+            t1 = _str_or_sqlrepr(j.table1, db)
             if t1 in tables: tables.remove(t1)
+            t2 = _str_or_sqlrepr(j.table2, db)
             if t2 in tables: tables.remove(t2)
         if tables:
             select += " FROM %s" % ", ".join(tables)
@@ -665,14 +675,14 @@ class Select(SQLExpression):
             select += join_str
 
         if self.ops['clause'] is not NoDefault:
-            select += " WHERE %s" % sqlrepr(self.ops['clause'], db)
+            select += " WHERE %s" % _str_or_sqlrepr(self.ops['clause'], db)
         if self.ops['groupBy'] is not NoDefault:
-            groupBy = sqlrepr(self.ops['groupBy'], db)
+            groupBy = _str_or_sqlrepr(self.ops['groupBy'], db)
             if isinstance(self.ops['groupBy'], (list, tuple)):
                 groupBy = groupBy[1:-1] # Remove parens
             select += " GROUP BY %s" % groupBy
         if self.ops['having'] is not NoDefault:
-            select += " HAVING %s" % sqlrepr(self.ops['having'], db)
+            select += " HAVING %s" % _str_or_sqlrepr(self.ops['having'], db)
         if self.ops['orderBy'] is not NoDefault and self.ops['orderBy'] is not None:
             orderBy = self.ops['orderBy']
             if self.ops['reversed']:
@@ -680,9 +690,9 @@ class Select(SQLExpression):
             else:
                 reverser = lambda x: x
             if isinstance(orderBy, (list, tuple)):
-                select += " ORDER BY %s" % ", ".join([sqlrepr(reverser(x), db) for x in orderBy])
+                select += " ORDER BY %s" % ", ".join([_str_or_sqlrepr(reverser(x), db) for x in orderBy])
             else:
-                select += " ORDER BY %s" % sqlrepr(reverser(orderBy), db)
+                select += " ORDER BY %s" % _str_or_sqlrepr(reverser(orderBy), db)
         start, end = self.ops['start'], self.ops['end']
         if self.ops['limit'] is not NoDefault:
             end = start + self.ops['limit']
@@ -765,7 +775,7 @@ class Update(SQLExpression):
                     update += ","
                 update += " %s=%s" % (key, sqlrepr(value, db))
         if self.whereClause is not NoDefault:
-            update += " WHERE %s" % sqlrepr(self.whereClause, db)
+            update += " WHERE %s" % _str_or_sqlrepr(self.whereClause, db)
         return update
     def sqlName(self):
         return "UPDATE"
@@ -784,8 +794,7 @@ class Delete(SQLExpression):
         whereClause = self.whereClause
         if whereClause is None:
             return "DELETE FROM %s" % self.table
-        if isinstance(whereClause, SQLExpression):
-            whereClause = sqlrepr(whereClause, db)
+        whereClause = _str_or_sqlrepr(whereClause, db)
         return "DELETE FROM %s WHERE %s" % (self.table, whereClause)
 
 registerConverter(Delete, SQLExprConverter)
@@ -837,6 +846,11 @@ def _IN(item, list):
     return SQLOp("IN", item, list)
 
 def IN(item, list):
+    from sresults import SelectResults # Import here to avoid circular import
+    if isinstance(list, SelectResults):
+        query = list.queryForSelect()
+        query.ops['items'] = [list.sourceClass.q.id]
+        list = query
     if isinstance(list, Select):
         return INSubquery(item, list)
     else:
@@ -896,23 +910,23 @@ class _LikeQuoted:
         if isinstance(s, SQLExpression):
             values = []
             if self.prefix:
-                values.append("'%s'" % self.prefix)
+                values.append(quote_str(self.prefix, db))
             s = _quote_like_special(sqlrepr(s, db), db)
             values.append(s)
             if self.postfix:
-                values.append("'%s'" % self.postfix)
+                values.append(quote_str(self.postfix, db))
             if db == "mysql":
                 return "CONCAT(%s)" % ", ".join(values)
             else:
                 return " || ".join(values)
         elif isinstance(s, basestring):
-            s = _quote_like_special(sqlrepr(s, db)[1:-1], db)
-            return "'%s%s%s'" % (self.prefix, s, self.postfix)
+            s = _quote_like_special(unquote_str(sqlrepr(s, db)), db)
+            return quote_str("%s%s%s" % (self.prefix, s, self.postfix), db)
         else:
            raise TypeError, "expected str, unicode or SQLExpression, got %s" % type(s)
 
 def _quote_like_special(s, db):
-    if db == 'postgres':
+    if db in ('postgres', 'rdbhost'):
         escape = r'\\'
     else:
         escape = '\\'
@@ -1121,22 +1135,13 @@ def FULLOUTERJOINUsing(table1, table2, using_columns):
 ## Subqueries (subselects)
 ########################################
 
-class OuterField(Field):
+class OuterField(SQLObjectField):
     def tablesUsedImmediate(self):
         return []
 
-class OuterTable(Table):
+class OuterTable(SQLObjectTable):
     FieldClass = OuterField
 
-    def __init__(self, table):
-        if hasattr(table, "sqlmeta"):
-            tableName = table.sqlmeta.table
-        else:
-            tableName = table
-            table = None
-        Table.__init__(self, tableName)
-        self.table = table
-
 class Outer:
     def __init__(self, table):
         self.q = OuterTable(table)
@@ -1178,6 +1183,7 @@ class RLIKE(LIKE):
         'maxdb': 'RLIKE',
         'mysql': 'RLIKE',
         'postgres': '~',
+        'rdbhost': '~',
         'sqlite': 'REGEXP'
     }
 
@@ -1246,7 +1252,7 @@ class ImportProxy(SQLExpression):
         self.sqlmeta = _Delay_proxy(table=_DelayClass(self, clsName))
         self.q = self
         self.soClass = None
-        classregistry.registry(registry).addClassCallback(clsName,lambda foreign, me: setattr(me, 'soClass', foreign), self)
+        classregistry.registry(registry).addClassCallback(clsName, lambda foreign, me: setattr(me, 'soClass', foreign), self)
 
     def __nonzero__(self):
         return True
