@@ -12,13 +12,13 @@ class Reparented1(SQLObject):
 
 class Reparented2(SQLObject):
     class sqlmeta(object):
+        @classmethod
         def setClass(cls, soClass):
             # Well, it's pretty hard to call the superclass method
             # when it's a classmethod and it's not actually your
             # *current* superclass.  Sigh
             real_sqlmeta.setClass.im_func(cls, soClass)
             cls.worked = True
-        setClass = classmethod(setClass)
 
     dummy = StringCol()
 
